class ez5.PdfCreator.Header extends CUI.Element

	initOpts: ->
		super()
		@addOpts
			data:
				check: "PlainObject"
				default: {}

	readOpts: ->
		super()
		@__data = @_data
		for position in ["left", "center", "right"]
			if @__data[position]
				continue
			@__data[position] = {}
		return @

	getData: ->
		return @__data

	renderPdfContent: (opts) ->
		header = CUI.dom.div("pdf-node-#{@constructor.name.toLowerCase()}")

		if not opts.preview
			# Specific measures for printing.
			CUI.dom.setStyle(header,
				"font-size": "8px"
				"padding-left": "0.28in"
				"padding-right": "0.28in"
				"width": "calc(100% - 0.56in)"
			)

		data = @getData()
		for position in ["left", "center", "right"]
			div = CUI.dom.div()
			CUI.dom.append(header, div)

			CUI.dom.setStyle(div,
				"display": "inline-block"
				"text-align": position
				"min-height": if opts.preview then "12px" else "8px"
			)

			_data = data[position]
			if _data.type == "logo"
				if not ez5.PdfCreator.logoImage
					continue

				CUI.dom.setStyle(div,"min-height": 0)

				# For now we use the base config logo in Base64.
				img = CUI.dom.element("img", src: ez5.PdfCreator.logoImage)
				imgDiv = CUI.dom.div("pdf-node-header-image-#{position}")
				CUI.dom.append(imgDiv, img)

				if not opts.preview
					# Specific measures for printing.
					CUI.dom.setStyle(img,
						"height": "0.3in"
					)
					CUI.dom.setStyle(imgDiv,
						"position": "absolute"
						"margin-top": "-0.21in"
					)
					if position == "right"
						# For some reason the right space is bigger in the print, that the reason of these magic numbers.
						# Also, landscape is bigger. Maybe it is a bug of headless chrome, but the right size should be 0.4in.
						styleRight = if opts.documentNode.isLandscape() then 3.20 else 2.35
						CUI.dom.setStyle(imgDiv,
							"right": "#{styleRight}in"
						)

				CUI.dom.append(div, imgDiv)
			else
				text = ez5.loca.getBestFrontendValue(_data.text)
				if not text
					continue

				CUI.dom.setStyle(div,
					"white-space": "nowrap"
					"overflow": "hidden"
				)

				text = text.replace(/%pageNumber%/g, "<span class='pageNumber'>1</span>")
				text = text.replace(/%totalPages%/g, "<span class='totalPages'>#{opts.objects?.length or 1}</span>")

				object = opts.objects[0] # Use the first object, even if it is multiple objects (grid)
				text = ez5.PdfCreatorUtils.replaceTextPlaceholders(text, object)

				textNodes = CUI.dom.htmlToNodes(text)
				CUI.dom.append(div, textNodes)

		# When the center div is empty, left and right widths are 50%, otherwise 33%.
		centerDiv = header.children[1]
		if not centerDiv.hasChildNodes()
			centerDiv.remove()

		children = CUI.dom.children(header)
		width = 100 / children.length
		for node in children
			CUI.dom.setStyle(node, "width": "#{width}%")
		return header

	getSettingsFields: (opts = {}) ->
		getFields = (position) =>
			textHintButton = new CUI.Button
				text: $$("pdf-creator.settings.node.header.text|hint")
				appearance: "flat"
				onClick: =>
					pop = new ez5.HintPopover
						element: textHintButton
						content: new CUI.Label(text: $$("pdf-creator.settings.node.header.text.popover"), markdown: true)
						padded: true
					pop.show()

			textField =
				type: CUI.MultiInput
				form:
					label: $$("pdf-creator.settings.node.header.text|label")
					hint: textHintButton
				control: ez5.loca.getLanguageControl()
				name: "text"

			if position == "center" or not @__supportsLogo()
				return [textField]

			onDataChanged = (data, field) ->
				form = field.getForm()
				textField = form.getFieldsByName("text")[0]
				if data.type == "logo"
					textField.hide(true)
				else
					textField.show(true)
				return

			logoFieldDisabled = not ez5.PdfCreator.logoImage
			if logoFieldDisabled
				logoText = $$("pdf-creator.settings.node.header.select.option-logo-disabled|label")
			else
				logoText = $$("pdf-creator.settings.node.header.select.option-logo|label")

			formFields = [
				type: CUI.Select
				name: "type"
				form:
					label: $$("pdf-creator.settings.node.header.select|label")
				options: [
					text: $$("pdf-creator.settings.node.header.select.option-text|label")
					value: "text"
				,
					text: logoText
					value: "logo"
					disabled: logoFieldDisabled
				]
				onDataInit: (field, data) -> onDataChanged(data, field)
				onDataChanged: onDataChanged
			,
				textField
			]

			return formFields

		forms = []
		for position in ["left", "center", "right"]
			forms.push
				type: CUI.Form
				form: label: $$("pdf-creator.settings.node.header.title.#{position}|label")
				render_as_block: true
				name: position
				fields: getFields(position)
				data: @getData()

		return forms

	# Enable/Disable logo in the settings.
	__supportsLogo: ->
		return true

	getSaveData: ->
		return CUI.util.copyObject(@getData(), true)